//Displays temperature of ds18B20 sensor. Draws thermometer as a bitmap from sd card.
//Draws a plot of temperature

#define DIR 2
//#include "thermometer.h"
#include <OSD50HD.h>
#include <SdFat.h>
#include <SdFatUtil.h>
#include <SPI.h>
#include <avr/wdt.h>

#include <OneWire.h>
#include <DallasTemperature.h>

OneWire  ds(7);

Sd2Card card;
SdVolume volume;
SdFile root;
File file;
OSD50HD osd;
unsigned char buf[64];
SdFat SD;
void setup() {
  delay(8000); //8000

  Serial.begin (115200);
  pinMode(DIR, OUTPUT);
  digitalWrite(DIR, LOW);

  SdCardInit();

  delay(40);
  osd.SendGrid(1, 25, 76, 386, 256, 1, 40, 20, 1, 30); //Net settings
  delay(40);
  osd.SendLineHV(1, 20, 70, 376, 230, 2, 0, 15); //Net background
  delay(40);
  osd.SetPlot(1, 24, 85, 22, 20, 3, 1, 28); //Plot settings
  delay(40);
  osd.SendText(1, 2, 9, "Temperature plot");

  osd.SendCircle(1, 80, 610, 70, 75, 75, 2, 0, 8);//background of thermometer
  delay(40);
  osd.SendLineHV(1, 35, 600, 120, 430, 2, 0, 12); //background of thermometer
  delay(40);
  int dim = 2;
  osd.SetBitmap(1, 40, 585, 255, 255, dim , 0, 1, 38);
  //  osd.SetBitmap(255, 585, 255, 255, 2 , 0, 1, 39);
  BitmapFromFile(0, "therm.bmp", dim);
}
void loop() {
  TemperatureMeasure();
}
void SdCardInit(void)
{
  PgmPrintln("Card inserted. Initialisation...");
  if (!card.init(SPI_FULL_SPEED, 10))
  {
    PgmPrintln("card.init failed!");
    return;
  }
  if (!volume.init(&card)) PgmPrintln("vol.init failed!");
  PgmPrint("Volume is FAT");

  if (!root.openRoot(&volume)) PgmPrintln("openRoot failed");
  PgmPrintln("Files found in root:");
  root.ls(LS_DATE | LS_SIZE);

  // Recursive list of all directories
  PgmPrintln("Files found in all dirs:");
  root.ls(LS_R);

  PgmPrintln("Done");
}
void BitmapFromFile(unsigned char NrBmp, char* str, int dim)
{
  file.open(&root, str, O_READ);
  uint32_t size = file.fileSize();
  size -= 62; //wycięcie naglowka

  //Sending bitmap from file

  unsigned char b[10] = {HEADER, ID, UPGRADEBITMAP, 0, 0, 0, 0, 0, 0, 0};
  int cs = 0;
  b[3] = cs; //
  int adrress = 0;
  char*y = (char*)&adrress;
  b[4] = y[1];
  b[5] = y[0];
  y = (char*)&size; //sending lenght of the file
  b[6] = y[0];
  b[7] = y[1];
  b[8] = NrBmp; //bitmap number

  osd.SendFrame(b, 9);

  uint32_t FilePtr = 0;
  int BufSize = 512;
  uint16_t line = 1;
  line = (dim + 1) * 4;
  const uint16_t count[40] = {1280, 640, 426, 320, 256, 213, 182, 160, 142, 128, 116, 106, 98, 91, 85, 80, 75, 71, 67, 64, 60,
                              58, 55, 53, 51, 49, 47, 45, 44, 42, 41, 40, 38, 37, 36, 35, 34, 33, 32, 32
                             };

  file.read(buf, 62); //Header reading
  for (int i = 0; i < count[dim]; i++) //426
  {
    file.read(buf, line);
    for (int t = 0; t < line; t++)
      Serial.write(buf[line - t - 1]);
  }
  file.close();
}
void TemperatureMeasure(void)
{
  static byte i, conversion = 0;
  static byte present = 0;
  static  byte data[12];
  static byte addr[8];
  static unsigned char tab[20];
  static uint32_t tit = 0, TWT = 0;
  if (millis() - tit > TWT)
  {
    if (conversion == 0)
    {
      ds.search(addr);
      ds.reset_search();
      if ( OneWire::crc8( addr, 7) != addr[7]) {
        return;
      }

      ds.reset();
      ds.select(addr);
      ds.write(0x44, 1);        // start conversion
      conversion = 1;
      TWT = 800;
      tit = millis();
    }
    else if (conversion == 1)
    {
      present = ds.reset();
      ds.select(addr);
      ds.write(0xBE);         // Read Scratchpad

      for ( i = 0; i < 9; i++) {           // we need 9 bytes
        data[i] = ds.read();
      }
      int HighByte, LowByte;
      LowByte = data[0];
      HighByte = data[1];
      float t = (HighByte << 8) + LowByte;
      t /= 16;
      char tmp[10];
      dtostrf(t, 1, 2, tmp);
      osd.SendText(1, 1, 22, tmp);
      int tt = - t * 4;

      osd.SendLineHV(1, 64, 833 + tt, 10, 135 - tt, 1, 0, 14); //temperature bar
      tt *= 4;
      tt += 550;
      if (tt < 0)
        tt = 0;
      if (tt > 253)
        tt = 253;
      osd.UpdatePlot(1, 0, tt);
      conversion = 0;
      TWT = 10;
      tit = millis();
    }
  }
}
